SystemOrganization addCategory: #'Spy-Tests'!
SystemOrganization addCategory: #'Spy-XML-RPC'!
SystemOrganization addCategory: #'Spy-XML-RPC-Test'!

TestCase subclass: #SpyTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spy-Tests'!

!SpyTest commentStamp: '<historical>' prior: 0!
Unit test framework.!

!SpyTest methodsFor: 'debugging' stamp: 'dls 11/12/2002 20:09'!
assertEquals: ob to: other msg: msg 
	"Make sure the two objects are equal."
	ob = other
		ifFalse: [self signalFailure: msg , ' (expected ' , ob asString , ', got ' , other asString , ')']! !

!SpyTest methodsFor: 'debugging' stamp: 'dls 11/12/2002 20:09'!
assertNil: ob withMessage: msg 
	"Verify the provided object is nil."
	ob
		ifNotNil: [self signalFailure: msg , ' (expected nil, wasn''t)']! !

!SpyTest methodsFor: 'debugging' stamp: 'dls 11/12/2002 20:09'!
assertNotNil: ob withMessage: msg 
	"Verify the provided objects is not nil."
	ob
		ifNil: [self signalFailure: msg , ' (expected not nil, was)']! !

!SpyTest methodsFor: 'debugging' stamp: 'dls 11/12/2002 20:09'!
assertSame: ob to: otherOb withMessage: msg 
	"Verify the two objects represent the same instance."
	ob == otherOb
		ifFalse: [self signalFailure: msg , ' (expected ' , ob asString , ', got ' , otherOb asString , ')']! !

SpyTest subclass: #XMLRPCDecoderTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spy-XML-RPC-Test'!

!XMLRPCDecoderTest commentStamp: 'dls 11/12/2002 20:14' prior: 0!
Test the XMLRPC Decoder.

XMLRPCDecoderTest new testDecoder!

!XMLRPCDecoderTest methodsFor: 'Testing' stamp: 'dls 11/15/2002 01:34'!
suite
	^ (TestSuite named: 'XMLRPCDecoderTests')
		addTest: (XMLRPCDecoderTest selector: #testDecoder)! !

!XMLRPCDecoderTest methodsFor: 'Testing' stamp: 'dls 11/15/2002 02:32'!
testDecoder
	"Test the decoder."
	| e d coll dict |
	e _ XMLRPCEncoder new.
	d _ XMLRPCDecoder new.
	dict _ Dictionary new.
	dict at: 'a' put: 'A'.
	dict at: 'b' put: 'B'.
	coll _ OrderedCollection new.
	coll add: 17.
	coll add: 'Some String'.
	coll add: true.
	coll add: false.
	coll add: 3.14159265359.
	coll add: XMLRPCDateTime now.
	coll add: #(1 2 3 'ima g' ).
	coll add: dict.
	coll
		do: [:item | 
			| xdom er dr | 
			er _ self
						xmlIfy: (e encode: item).
			xdom _ XMLDOMParser parseDocumentFrom: er readStream.
			dr _ d
						decode: (xdom elements at: 1).
			"Transcript show: 'Decoded ' , er , ' to ' , dr asString. 
			Transcript cr."
			self
				assertEquals: item
				to: dr
				msg: 'Decoding ' , item className]! !

!XMLRPCDecoderTest methodsFor: 'private' stamp: 'dls 11/11/2002 23:53'!
xmlIfy: t 
	^ '<?xml version="1.0"?><param>' , t , '</param>'! !

SpyTest subclass: #XMLRPCEncoderTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spy-XML-RPC-Test'!

!XMLRPCEncoderTest commentStamp: 'dls 11/12/2002 20:14' prior: 0!
Test the XMLRPC Encoder.

XMLRPCEncoderTest new testEncoder!

!XMLRPCEncoderTest methodsFor: 'Testing' stamp: 'dls 11/15/2002 01:34'!
suite
	^ (TestSuite named: 'XMLRPCEncoderTests')
		addTest: (XMLRPCEncoderTest selector: #testEncoder)! !

!XMLRPCEncoderTest methodsFor: 'Testing' stamp: 'dls 11/15/2002 22:30'!
testEncoder
	"Run all the encoder tests."
	| e d |
	e _ XMLRPCEncoder new.
	"Transcript show: 'Running encoder test.'; cr."
	self
		assertEquals: '<value><i4>17</i4></value>'
		to: (e encode: 17)
		msg: 'Encoding int.'.
	self
		assertEquals: '<value><double>3.14159265358979</double></value>'
		to: (e encode: 3.14159265358979)
		msg: 'Encoding float.'.
	self
		assertEquals: '<value><string>pi</string></value>'
		to: (e encode: 'pi')
		msg: 'Encoding string.'.
	self
		assertEquals: '<value><boolean>1</boolean></value>'
		to: (e encode: True)
		msg: 'Encoding True'.
	self
		assertEquals: '<value><boolean>0</boolean></value>'
		to: (e encode: False)
		msg: 'Encoding False'.
	self
		assertEquals: '<value><array><data><value><i4>1</i4></value><value><i4>2</i4></value></data></array></value>'
		to: (e encode: #(1 2 ))
		msg: 'Encoding array.'.
	d _ Dictionary new.
	d at: 'a' put: 'A'.
	d at: 'b' put: 'B'.
	self
		assertEquals: '<value><struct><member><name>b</name><value><string>B</string></value></member><member><name>a</name><value><string>A</string></value></member></struct></value>'
		to: (e encode: d)
		msg: 'Encoding struct.'! !

SpyTest subclass: #XMLRPCTest
	instanceVariableNames: 'proxy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spy-XML-RPC-Test'!

!XMLRPCTest methodsFor: 'Testing' stamp: 'dls 11/15/2002 22:30'!
performCall: method arg: a 
	"Perform method invocation and make sure the result is the same as the  
	passed-in value."
	| r p |
	p _ OrderedCollection new.
	p add: a.
	r _ proxy invokeMethod: method withArgs: p.
	"Transcript show: 'Invoked ' , method , ' with ' , a asString , ' which returned ' , r asString;
		 cr."
	self
		assertEquals: a
		to: r
		msg: 'Failed on method ' , method.
! !

!XMLRPCTest methodsFor: 'Testing' stamp: 'dls 11/15/2002 22:30'!
setUp
	"Set up the test suite:"
	| url |
	url _ Url absoluteFromText: 'http://bleu.west.spy.net/servlet/net.spy.rpc.XMLRPC'.
	proxy _ XMLRPCProxy new url: url.
	"Transcript show: 'Set up XMLRPCTest';
		 cr."

	^ self! !

!XMLRPCTest methodsFor: 'Testing' stamp: 'dls 11/15/2002 02:43'!
suite
	^ (TestSuite named: 'XMLRPCTests')
		addTest: (XMLRPCTest selector: #testInt);
		
		addTest: (XMLRPCTest selector: #testDouble);
		
		addTest: (XMLRPCTest selector: #testString);
		
		addTest: (XMLRPCTest selector: #testBoolean);
		
		addTest: (XMLRPCTest selector: #testArray);
		
		addTest: (XMLRPCTest selector: #testStruct);
		
		addTest: (XMLRPCTest selector: #testDate);

		addTest: (XMLRPCTest selector: #testBinary)! !

!XMLRPCTest methodsFor: 'Testing' stamp: 'dls 11/15/2002 01:37'!
testArray
	"Test arrays."
	self performCall: 'echo.echoArray' arg: #('A' 1 3 2.525)! !

!XMLRPCTest methodsFor: 'Testing' stamp: 'dls 11/15/2002 02:51'!
testBinary
	"Test Binary data."
	| r p msg |
	msg _ 'Test for base64.'.
	p _ OrderedCollection new.
	p add: msg readStream.
	r _ proxy invokeMethod: 'echo.echoByte' withArgs: p.

	self assertEquals: msg to: (r contents asString) msg: 'Binary (base64) method failed.'! !

!XMLRPCTest methodsFor: 'Testing' stamp: 'dls 11/15/2002 01:35'!
testBoolean
	"Test booleans."
	self performCall: 'echo.echoBoolean' arg: true.
	self performCall: 'echo.echoBoolean' arg: false! !

!XMLRPCTest methodsFor: 'Testing' stamp: 'dls 11/15/2002 02:43'!
testDateTime
	"Test DateTime."
	self performCall: 'echo.echoDate' arg: (XMLRPCDateTime now)! !

!XMLRPCTest methodsFor: 'Testing' stamp: 'dls 11/15/2002 00:47'!
testDouble
	"Test doubles."
	self performCall: 'echo.echoDouble' arg: 3.14159265358979! !

!XMLRPCTest methodsFor: 'Testing' stamp: 'dls 11/15/2002 00:39'!
testInt
	"Test ints."
	self performCall: 'echo.echoInt' arg: 19.! !

!XMLRPCTest methodsFor: 'Testing' stamp: 'dls 11/15/2002 00:50'!
testString
	"Test strings."
	self performCall: 'echo.echoString' arg: 'This is a test message.'.
	self performCall: 'echo.echoString' arg: 'Embedded XML <see>.'.! !

!XMLRPCTest methodsFor: 'Testing' stamp: 'dls 11/15/2002 01:53'!
testStruct
	"Test structs."
	| d |
	d _ Dictionary new.
	d at: 'a' put: 'Value for A'.
	d at: 'b' put: 'Value for B'.
	self performCall: 'echo.echoStruct' arg: d! !

TestCase subclass: #XMLRPCTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spy-XML-RPC-Test'!

!XMLRPCTests commentStamp: 'dls 11/12/2002 20:17' prior: 0!
SUnit tests for all XML-RPC functionality.!

!XMLRPCTests methodsFor: 'Testing' stamp: 'dls 11/12/2002 21:16'!
suite
	| s |
	s _ TestSuite named: 'All XML RPC Tests'.
	s
		addTest: (XMLRPCEncoderTest selector: #testEncoder).
	s
		addTest: (XMLRPCDecoderTest selector: #testDecoder).
	^ s! !

Error subclass: #XMLRPCException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spy-XML-RPC'!

!XMLRPCException commentStamp: 'dls 11/15/2002 03:05' prior: 0!
Exception thrown when unexpected events occur during XML-RPC processing.!

XMLRPCException subclass: #XMLRPCFaultException
	instanceVariableNames: 'faultCode faultMessage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spy-XML-RPC'!

!XMLRPCFaultException commentStamp: 'dls 11/15/2002 03:05' prior: 0!
Exception thrown to indicate an XML-RPC fault.!

!XMLRPCFaultException classSide methodsFor: 'signaling' stamp: 'dls 11/15/2002 22:19'!
signal: faultCode withMessage: faultMessage 
	"Signal an XMLRPC fault with the given fault code and message."
	XMLRPCFaultException new faultCode: faultCode ; faultMessage: faultMessage ; signal: faultMessage! !

!XMLRPCFaultException methodsFor: 'accessing' stamp: 'dls 11/15/2002 22:17'!
faultCode
	"Get the fault code."
	^ faultCode! !

!XMLRPCFaultException methodsFor: 'accessing' stamp: 'dls 11/15/2002 22:17'!
faultCode: to
	"Set the fault code to the given value."
	faultCode _ to! !

!XMLRPCFaultException methodsFor: 'accessing' stamp: 'dls 11/15/2002 22:18'!
faultMessage
	"Get the fault message."
	^ faultMessage! !

!XMLRPCFaultException methodsFor: 'accessing' stamp: 'dls 11/15/2002 22:18'!
faultMessage: to
	"Set the fault message to the given value."
	faultMessage _ to! !

Object subclass: #XMLRPCDateTime
	instanceVariableNames: 'date time'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spy-XML-RPC'!

!XMLRPCDateTime commentStamp: 'dls 11/15/2002 02:56' prior: 0!
DateTime object for sending and receiving dates via XML-RPC.

	See the convenience class methods for easily creating instances of this class.!

!XMLRPCDateTime classSide methodsFor: 'instance creation' stamp: 'dls 11/15/2002 02:23'!
fromDate: d time: t 
	"Get a new XMLRPCDateTime instance from the given date and time."
	^ XMLRPCDateTime new date: d;
		 time: t! !

!XMLRPCDateTime classSide methodsFor: 'instance creation' stamp: 'dls 11/15/2002 02:30'!
fromISO8601String: s
	"Get a new XMLRPCDateTime instance from the given ISO8601 string."
	^ XMLRPCDateTime new decodeISO8601: s! !

!XMLRPCDateTime classSide methodsFor: 'instance creation' stamp: 'dls 11/15/2002 02:31'!
now 
	"Get a new XMLRPCDateTime instance representing the current time."
	^ XMLRPCDateTime new date: (Date today); time: (Time now)! !

!XMLRPCDateTime methodsFor: 'private' stamp: 'dls 11/18/2002 22:55'!
= anObject
	"True if the other object is an XMLRPCDateTime representing the same date and time."

	| rv |
	rv _ false.

	((self class) == (anObject class))
		ifTrue: [ rv _ ( (self date) = (anObject date) ) & ((self time) = (anObject time)) ].

	^ rv ! !

!XMLRPCDateTime methodsFor: 'accessing' stamp: 'dls 11/15/2002 02:00'!
date
	"Get the date portion of the datetime."
	^ date! !

!XMLRPCDateTime methodsFor: 'accessing' stamp: 'dls 11/15/2002 02:01'!
date: d
	"Set the date portion of the datetime."
	date _ d! !

!XMLRPCDateTime methodsFor: 'xmlrpc' stamp: 'dls 11/15/2002 02:28'!
decodeISO8601: s 
	"Set the date and time in this object by parsing the ISO8601 string."
	| d t Y M D |
	Y _ (s copyFrom: 1 to: 4) asInteger.
	M _ (s copyFrom: 5 to: 6) asInteger.
	D _ (s copyFrom: 7 to: 8) asInteger.
	d _ Date
				newDay: D
				month: M
				year: Y.
	t _ Time readFrom: ((s copyFrom: 10 to: 17) readStream).
	self date: d.
	self time: t! !

!XMLRPCDateTime methodsFor: 'xmlrpc' stamp: 'dls 11/15/2002 02:19'!
encodeISO8601
	| rv |
	"Encode me into an ISO8601 datetime for XML-RPC"
	rv _ Text new.

	rv append: (date year asString).
	rv append: (self pad: (date monthIndex asString) to: 2 with: '0').
	rv append: (self pad: (date dayOfMonth asString) to: 2 with: '0').
	rv append: 'T'.
	rv append: (time print24).

	^ rv asString! !

!XMLRPCDateTime methodsFor: 'private' stamp: 'dls 11/18/2002 23:01'!
hash
	"Predictable hash for this object."
	^ date hash bitXor: (time hash)! !

!XMLRPCDateTime methodsFor: 'private' stamp: 'dls 11/15/2002 02:22'!
pad: in to: preferredSize with: thing 
	"Return a string with in padded on the left with the given number of  
	the given thing."
	| rv |
	rv _ Text new.
	rv append: in asString.
	[rv size < preferredSize]
		whileTrue: [rv prepend: thing].
	^ rv asString! !

!XMLRPCDateTime methodsFor: 'printing' stamp: 'dls 11/15/2002 02:55'!
printOn: aStream
	"Send it to the stream."
	aStream nextPutAll: (self encodeISO8601).! !

!XMLRPCDateTime methodsFor: 'accessing' stamp: 'dls 11/15/2002 02:01'!
time
	"Get the time portion of the datetime."
	^ time! !

!XMLRPCDateTime methodsFor: 'accessing' stamp: 'dls 11/15/2002 02:00'!
time: t
	"set the time portion of the datetime."
	time _ t! !

Object subclass: #XMLRPCDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spy-XML-RPC'!

!XMLRPCDecoder commentStamp: 'dls 11/15/2002 02:58' prior: 0!
XML-RPC decoder.

This class will take XML-RPC results and translate them into smalltalk types.  The following translations will take place:

	boolean  -> True or False
	string    -> String
	double   -> Float class
	i4         -> SmallInteger
	struct    -> Dictionary
	array    -> Array
	dateTime.iso8601 -> XMLRPCDateTime
	base64   -> Stream!

!XMLRPCDecoder methodsFor: 'decoding' stamp: 'dls 11/18/2002 22:38'!
decode: xml 
	"Decode an XML Element representing an XML-RPC response."
	"xml inspect."
	| el0 |
	xml elements isEmpty
		ifTrue: [^ xml contentString].
	"Figure out what to do based on the name of the element (el0 is usually  
	a required argument."
	el0 _ xml elements at: 1.
	(el0 name) caseOf: {
		[#array] -> [^ self decodeArray: el0].
		[#struct] -> [^ self decodeStruct: el0].
		[#param] -> [^ self decode: el0].
		[#value] -> [^ self decode: el0].
		[#string] -> [^ el0 contentString].
		[#i4] -> [^ SmallInteger readFrom: el0 contentString readStream].
		[#int] -> [^ SmallInteger readFrom: el0 contentString readStream].
		[#double] -> [^ Float readFrom: el0 contentString readStream].
		[#boolean] -> [^ el0 contentString = '1'].
		[#'dateTime.iso8601'] -> [^ XMLRPCDateTime fromISO8601String: el0 contentString].
		[#base64] -> [^ Base64MimeConverter mimeDecodeToBytes: el0 contentString readStream].
	} otherwise: [XMLRPCException signal: 'Unhandled type!!  ' , xml name.]! !

!XMLRPCDecoder methodsFor: 'private' stamp: 'dls 11/12/2002 00:20'!
decodeArray: xml 
	"Decode the Array."
	| rv |
	rv _ OrderedCollection new.
	(xml elements at: 1) elements
		do: [:el | rv
				add: (self decode: el)].
	^ rv asArray! !

!XMLRPCDecoder methodsFor: 'private' stamp: 'dls 11/14/2002 22:43'!
decodeStruct: xml 
	"Decode the Struct."
	| rv |
	rv _ Dictionary new.
	xml elements
		do: [:el | rv
				at: (el elementAt: 'name') contentString
				put: (self
						decode: (el elementAt: 'value'))].
	^ rv! !

Object subclass: #XMLRPCEncoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spy-XML-RPC'!

!XMLRPCEncoder commentStamp: 'GermanArduino 6/6/2010 17:50' prior: 0!
Encode objects into proper XML-RPC types.

The following types are understood (smalltalk types in parens):

	Booleans (True and False)
	Strings (String)
	Strings (ByteString) --> GermanArduino 06/06/2010
	Doubles (Float class)
	Integers (SmallInteger class)
	Struct (Dictionary)
	Array (Array)
	dateTime.iso8601 (XMLRPCDateTime)
	base64 (Stream)!

!XMLRPCEncoder methodsFor: 'encoding' stamp: 'GermanArduino 6/6/2010 17:34'!
encode: val 
	"Encode the given value for inclusion in an XML-RPC request or  
	response.  
	 
	Returns the encoded value.  
	"
	val caseOf: {
		[True] -> [^ self wrapVal: '1' type: 'boolean'].
		[true] -> [^ self wrapVal: '1' type: 'boolean'].
		[False] -> [^ self wrapVal: '0' type: 'boolean'].
		[false] -> [^ self wrapVal: '0' type: 'boolean'].
	} otherwise: [
		(val class) caseOf: {
			[SmallInteger] -> [^ self wrapVal: val asString type: 'i4'].
			[Float] -> [^ self wrapVal: val asString type: 'double'].
			[String] -> [^ self
				wrapVal: (self normalize: val)].
			[ByteString] -> [^ self
				wrapVal: (self normalize: val)].
			[XMLRPCDateTime] -> [^ self wrapVal: val encodeISO8601 type: 'dateTime.iso8601'].
			[Array] -> [^ self
				wrapVal: (self encodeArray: val)
				type: 'array'].
			[Dictionary] -> [^ self
				wrapVal: (self encodeStruct: val)
				type: 'struct'].
		} otherwise: [
			val isStream
				ifTrue: [^ self wrapVal: (Base64MimeConverter mimeEncode: val) contents type: 'base64']
				ifFalse:
					"Unhandled type."
					[XMLRPCException signal: 'Unhandled type:  ' , val className].
			]]! !

!XMLRPCEncoder methodsFor: 'private' stamp: 'dls 11/11/2002 23:04'!
encodeArray: a 
	"Encode an array."
	| rv |
	rv _ Text new.
	rv append: '<data>'.
	a
		do: [:el | rv
				append: (self encode: el)].
	rv append: '</data>'.
	^ rv asString! !

!XMLRPCEncoder methodsFor: 'private' stamp: 'dls 11/15/2002 00:59'!
encodeStruct: s 
	"Encode a struct."
	| rv |
	rv _ Text new.
	s
		keysAndValuesDo: [:key :val | rv append: '<member><name>' , (self normalize: key) , '</name>'
					, (self encode: val) , '</member>'].
	^ rv asString! !

!XMLRPCEncoder methodsFor: 'private' stamp: 'dls 11/15/2002 01:33'!
normalize: s 
	"Normalize an XML string."
	| rv map |
	map _ Dictionary new.
	map at: $< put: '&lt;'.
	map at: $> put: '&gt;'.
	map at: $& put: '&amp;'.
	rv _ Text new.
	s
		do: [:c | 
			rv
				append: (map
						at: c
						ifAbsent: [c])].
	^ rv asString! !

!XMLRPCEncoder methodsFor: 'private' stamp: 'dls 11/11/2002 21:16'!
wrapVal: val
	"Wrap the given value in <value> tags, assuming it's already a string."
	^ self wrapVal: val type: 'string'! !

!XMLRPCEncoder methodsFor: 'private' stamp: 'dls 11/11/2002 23:02'!
wrapVal: val type: t 
	"Wrap the given value in <value> and <t>"
	^ '<value><' , t , '>' , val , '</' , t , '></value>'! !

Object subclass: #XMLRPCProxy
	instanceVariableNames: 'url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spy-XML-RPC'!

!XMLRPCProxy commentStamp: 'dls 11/18/2002 23:06' prior: 0!
Proxy class to issue XML-RPC requests.

	Example (invoking a method that takes a single integer argument):

	u _ Url absoluteFromText: 'http://bleu.west.spy.net/servlet/net.spy.rpc.XMLRPC'.
	proxy _ XMLRPCProxy withUrl: u.
	proxy invokeMethod: 'echo.echoInt' withArgs: #(1).!

!XMLRPCProxy classSide methodsFor: 'instance creation' stamp: 'dls 11/15/2002 03:01'!
withUrl: u
	"Get a new instance of XMLRPCProxy to connect to a given URL."
	^ XMLRPCProxy new url: u! !

!XMLRPCProxy methodsFor: 'private' stamp: 'GermanArduino 6/10/2010 08:26'!
buildXmlRpc: method withArgs: args 
	"Build the XMLRPC data required to invoke the given method with the 
	given collection of arguments."
	| params e rv |
	e _ XMLRPCEncoder new.
	params _ Text new.
	args
		do: [:p | params append: '<param>'
					, (e encode: p) , '</param>'].
	rv _ '<?xml version="1.0"?><methodCall><methodName>' , method , '</methodName><params>' , params , '</params></methodCall>'.

rv := '<?xml version="1.0"?><methodCall>
	<methodName>flickr.test.echo</methodName>
	<params>
		<param>
			<value>
				<struct>
					<member>
						<name>name</name>
						<value><string>garduino</string></value>
					</member>
					<member>
						<name>api_key</name>
						<value><string>b7d7df90f7c8ba3fd6f7012d5d5bc64c</string></value>
					</member>
				</struct>
			</value>
		</param>
	</params>
</methodCall>'.
	
	self halt.

	^ rv! !

!XMLRPCProxy methodsFor: 'invoking' stamp: 'dls 11/15/2002 22:25'!
invokeMethod: method 
	"Invoke a method on the XML-RPC server with no arguments."
	^ self invokeMethod: method withArgs: #()! !

!XMLRPCProxy methodsFor: 'invoking' stamp: 'dls 11/15/2002 22:27'!
invokeMethod: method withArgs: args 
	"Invoke a method on the XML-RPC server.

	args is a list of parameters to be passed.  For example, if the remote method is called ``test.Method'' and takes two ints, you'd invoke is like this:

	anXMLRPCProxy invokeMethod: 'test.method' withArgs: #(19 77)
"
	| req res rv |
	req _ self buildXmlRpc: method withArgs: args.
	res _ self sendXmlRpc: req.
	rv _ self processResponse: res.
	^ rv! !

!XMLRPCProxy methodsFor: 'private' stamp: 'dls 11/15/2002 00:23'!
processFault: xmlFault 
	"Process an XML-RPC fault into an XMLRPCFaultException"
	| d decoder |
	decoder _ XMLRPCDecoder new.
	d _ decoder decode: xmlFault.
	XMLRPCFaultException
		signal: (d at: 'faultCode')
		withMessage: (d at: 'faultString').! !

!XMLRPCProxy methodsFor: 'private' stamp: 'GermanArduino 6/10/2010 08:40'!
processResponse: xmlStream 
	"Process the results."
	| xmldoc mr fr pr |
	
	self halt.
	
	xmldoc _ XMLDOMParser parseDocumentFrom: xmlStream.
	mr _ xmldoc elementAt: 'methodResponse'.
	mr
		ifNil: [XMLRPCException signal: 'methodResponse not found in XML Stream.'].
	"See if it's a fault response."
	fr _ mr elementAt: 'fault'.
	fr
		ifNotNil: [self processFault: fr].
	"If we made it this far, it's a valid response, look for the results."
	pr _ mr elementAt: 'params'.
	pr
		ifNil: [XMLRPCException signal: 'Params not found in XML-RPC response.'].
	"Decode, and return the response."
	^ XMLRPCDecoder new decode: pr! !

!XMLRPCProxy methodsFor: 'private' stamp: 'CdG 12/18/2004 01:41'!
sendXmlRpc: data 
	"Send the XML-RPC data and get the response. Return the raw XML."
	| s req res xml |
	s _ HTTPSocket initHTTPSocket: url ifError: [:msg | XMLRPCException signal: 'Error connecting ' , url asString, '(', msg, ')'].
	req _ 'POST ' , url fullPath , ' HTTP/1.0' , String crlf , 'Host: ' , url authority , String crlf , 'Content-type: text/xml' , String crlf , 'Content-length:  ' , data size asString , String crlf , 'User-Agent:  Spy Internetworking XML-RPC client for Smalltalk' , String crlf , String crlf , data.
	"Send the command"
	s sendCommand: req.
	"Make sure the result was valid."
	s responseCode = 200
		ifFalse: ['XXX Raise exception.'].
	"Get the data up to the first double-newline (indicating end of headers)"
	res _ s getResponseUpTo: String crlf , String crlf.
	xml _ s getRestOfBuffer: (res at: 3) contents.
	^ xml! !

!XMLRPCProxy methodsFor: 'accessing' stamp: 'dls 11/14/2002 23:18'!
url
	"Get the URL against which requests will be invoked."
	^ url! !

!XMLRPCProxy methodsFor: 'initializing' stamp: 'dls 11/14/2002 23:17'!
url: aUrl
	"Set the URL that will be servicing the requests."
	url _ aUrl! !
